%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%                                                              %%%%%%
%%%%%%                           %%%%%%
%%%%%%                                                              %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------------------------------------------------%
%               Nonlinear Models of Financial Crises             %
%                    and Macroprudential Regulation                      %
%------------------------------------------------------------------------%

% Lab Session: Code for Bianchi, Liu and Mendoza (JIE 2016), updated 09/2021 
% Endowment Version with News on Endowment Shocks and Interest Rate Shocks
% Solves Decentralized Equilibrium and Social Planners problem
%% 0. Housekeeping
delete(gcp('nocreate'))
clear all;
clc;
close all

% Tips for using parallel toolbox: Depending on size of the grids of bonds
% and shocks and on which nonlinear solver is used, using the parallel toolbox
% can speed up execution significantly. Loops with the for statements 
% that do not need to be done in sequential order can be parallelized and, depending 
% on how many cores are in the computer, will run much faster. To do this,
% insert the parameter matlabpool(#) or parpool(#), where # is the number 
% of cores to use in the execution (set it to < than the # of cores in 
% your computer). Parpool is for newer matlab versions, older ones use
% matlabpool. Then change the for statements to parallelize to parfor.
% Parfor runs faster when using fsolve or fminbnd while it is about as fast
% as for when using fzero.
% Warning: Do this only with loops where the execution is not required to
% go in order. In this code, this can be done with four loops that can be 
% particularly slow, the ones setting the values of the emu, EE, emuSP 
% and EESP matrices. For these, you can use parfor for the long loop (i=1:NB). 
% Solutions are identical as using for but execution is faster.
% You may also need to increase the ClusterSize property setting in the 
% 'local' configuration using the Configurations Manager option in the
% Parallel menu (change it to your max # of cores and click on "validate
% configuration")

if verLessThan('matlab','8.2') %(comment this is in if using parallel toolbox)
	matlabpool open     %  Old version. Use matlabpool
else
	poolpbject=parpool(2)  % Newer version uses parpool(#) where # is number 
                           % available CPUS to use (<= # of CPUs in
                           % computer)
end

%% 1. Values of model and algorithm parameters

%%%%%%%%%%%%%%%%%  Model Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global omega sigma ita beta r Zprob  B  bp  c uptd Prob Q  
sigma=2; %coefficient of relative risk aversion
ita=(1/0.83)-1; %CES preferences exponent parameter (elasticity=1/(1+ita))
beta=0.91; %discount factor
omega=0.32; % CES preferences share parameter
kappa=0.32; %fraction of income pledgeable as collateral
rho=0.54; %autocorrelation of yT process
sigmay=0.059; %standard dev of yT process
NS=3; %nodes in the yT grid and news shock grid
NSS=NS*NS; %combined nodes of yT and news shocks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%  Algorithm parameters  %%%%%%%%%%%%%%%%%%%%%%%%%
uptd=0.2; %0.1; %Weight on new policy function in update to next iteration
%change to 0.1 or smaller (w # if policy function fluctuate)
outfreq=20; %Display frequency (shows in screen each 20th iteration)
iter_tol=2600;  % Maximum number of fixed-point iterations
tol=1e-7; %1e-4; % Convergence criterion (lower for session, higher for accuracy
tol_EEbind = 1.0e-16; % tolerance for binding constriant 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%% Precision of news shocks  %%%%%%%%%%%%%%%%%%%%
info=2/3; % =1 is perfect signal; =1/NS is useless signal (theta in the paper)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Two-point regime switching interest rate shocks%%%%%%%%%%%%%%%%
%%%%%%%%%% see calibration section in the paper %%%%%%%
Rlow=-0.03283215044;
Rhigh=0.014500;
Fll=0.6; %0.9;
Fhh=14/15; %59/60; 
r=[Rhigh,Rlow];
Rprob=[Fhh,1-Fhh;1-Fll,Fll]; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%% Bonds grid nodes and boundaries %%%%%%%%%%%%%%%%
NB=100; %number of nodes.
bmin=-1.11; %-1.06; 
bmax=-0.31; %-0.41;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%% Create bonds grid
B=zeros(NB,1);
B(1)=bmin;
%%%%%%%%%%%%%%% Symmetric  %%%%%%%%%%%%%%%%%%%%%
%%%%%%% (finer grid for low i to improve accuracy near lower bound)%%%%%
for i=2:NB;
    B(i)=B(i-1)+(bmax-B(i-1))/((NB-i+1)^1.05); 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% 2. Construction of Markov Chain

%%%%%%%%%%  Tauchen Hussey  (calls tauchenhussey subroutine)  %%%%%%%%%%%%%
[Z,Zprob] = tauchenhussey(NS,0,rho,sigmay,sigmay); 
% In Zprob rows are today, columns are tomorrow
Z=exp(Z);
ZZ=diag(Z);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%  Incorporate News Structure %%%%%%%%%%%%%%%%%%%%%%%
Psz=zeros(NS); %transitional probability of s_t given z_{t+1}: p(s|z') 
Psz=(1-info)/(NS-1).*(1-eye(NS))+eye(NS).*info;

% P(z'|s,z) is derived by bayes rule, as in Equation (1) in the handout

Pzsz=zeros(NSS,NS); 
for i=1:NS % s
    for j=1:NS % z
        for l=1:NS % z'
            k=(i-1)*NS+j;
            Pzsz(k,l)=(Psz(l,i)*Zprob(j,l))/(Zprob(j,:)*Psz(:,i));
        end
    end
end
% Order for Pzsz (column is current state): 
% z1'|s1z1; z1'|s1z2; ... z1'|s1zN; z1'|s2z1; 
% z1'|s2z2;... z1'|s2zN;... z2'|s1z1......z2'|s2z2;

% p(z's'|zs) as in equation (2) in the handout 
% (Equation (3) in Durdu et al (2013))
NProb=zeros(NSS,NSS);  
for i=1:NS % s
    for j=1:NS % z
        for l=1:NS % z'
            for k=1:NS %s'
                ii=(i-1)*NS+j; %today
                jj=(k-1)*NS+l; %tomorrow
                NProb(ii,jj)=Pzsz(ii,l)*((Zprob(l,:)*Psz(:,k)));
            end
        end
    end
end
%Order: s1z1, s1z2, s2z1, s2z2

%%%%%%% Construct Full Trans Matrix of Shocks (R, yT and news %%%%%%%%%%%%%
% This is the same as a kronecker product
Prob=[NProb*Rprob(1,1),NProb*Rprob(1,2);NProb*Rprob(2,1),NProb*Rprob(2,2)]; 
%Today is goods shock, bad to good and interest rate shock bad to good; 
State_Index=[1:length(Prob)]';



%% 3. Decentralized Equilibrium
%  3.1 Initial values 

% Recall we want policy functions for each pair (b,z) where b: current
% bonds, is the nedogenous state and z=(y^T,s,r) is the exogenous state.
% Hence our policies are matrices of dimension NB x Ntotal

Ntotal=NSS*length(r);           % Total Number of Exogenous States (s)
yt=repmat(Z',NB,1);                             
yt=repmat(yt,1,NS*length(r));   % Tradeables Output at each state (b,z)

R=[repmat(r(1),NB,NSS),repmat(r(2),NB,NSS)]; % Interest Rate at each (b,z)

Q=1./(1+R);                     % Bond price at each state (b,z)

b=repmat(B,1,Ntotal);           % Debt at each state (b,z)

yn=ones(NB,Ntotal);             % Constant Nontradables endowment yN=1

%%%%%%%%%%%%%%%%%%%  Step 1 of Cell 3 in the handout
%%%%%%%%%%Initial conjectures of decision rules 
%%%%%%(they are updated before the end of each iteration).
    
bp=b;    % Initial bonds dec rule: b'(b,z)=b (45degree line)
c=b+yt-bp.*Q; % Initial cT dec rule implied by initial bonds dec rule
price=(1-omega)/omega*c.^(1+ita); % Initial Nontradables price implied by cT
bpmax=-kappa*(price.*yn +yt)./Q; %-kappa*(price+yt);  % Debt Limit
cbind=b+yt-bpmax.*Q; % Initial constrained cT values

EE=zeros(NB,Ntotal);       % Initial Unconstrained Euler Equation Error
mup=ones(NB,Ntotal);       % Preallocation for marginal utility of tradables
emu=zeros(NB,Ntotal);      % Preallocation for expected marginal utility



%% 3.2 Fixed point Iteration Loop (see Cell 3, Step 2 in Handout)
iter=0;  % Initialize iteration counter
d2=100;  % Initialize value of convergence metric (just to enter loop)
options =optimset('Display','off'); %options for fzero nonlinear solver
tic
disp('DE Iter      Norm');

     %%%%%%%%Step 2 of Cell 3 in handout: 
while d2>tol && iter <iter_tol  %Exits loop if either convergence is attained
    % or total number of iterations exceed value of iter_tol parameter
    
%    oldp=price; %conjectured pN for current iteration
    oldc=c; % conjectures cT for current iteration
    oldbp=bp; %conjecture bonds dec rule for current iteration
    
   totalc=(omega*c.^(-ita)+(1-omega)*yn.^(-ita));   % (Total Consumption)^(-ita)
    % mup(b,z) is Marginal utility of C^T given current consumption policy c(b,z)
    mup=omega*totalc.^(sigma/ita).*(totalc.^(-1/ita-1)).*(c.^(-ita-1)); 
    
    parfor i=1:NB % Can be parallelized using parfor instear of for
        for j=1:Ntotal                
            %EMU is expected marginal utility tomorrow in today's grid, given
            %current future debt policy bp(b,z). Found by interpolation
            emu(i,j)=beta*(1+R(i,j))*interp1(B,mup,bp(i,j),'linear','extrap')*Prob(j,:)';
        end
    end
      
    parfor i=1:NB % % Can be parallelized using parfor instear of for
        for j=1:Ntotal  
      %%%%%%%%%%%% Step 3 of Cell 3 in the handout.
          % Calculate Euler Equation gap assuming the constraint binds
          % "today" (i.e. using cbind(b,z) for uT(t), this is eq. (4) in the handout. 
            EE(i,j)=(omega*cbind(i,j)^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cbind(i,j)^(-ita-1)-emu(i,j);
            cinit=c(i,j); %initial condition for solver  
     %%%%%%%%%%%% Step 4 of Cell 3 in the handout.
            if EE(i,j)>tol_EEbind %tol*1e-5	% If positive, constraint will be binding then:
                
                bp(i,j)=bpmax(i,j);  % bonds determined by binding credit constraint
                c(i,j)=cbind(i,j);   % consumption solved from budget constraint
                
            else % Constraint not binding
                
     %%%%%%% Step 5 of Cell 3 in the handout:
                % Define function used to find consumption decision
                % that solves Euler Equation 
               f = @(cc) (omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emu(i,j);  
                % Solve Euler Equation to get new consumption dec rule
                c0=cinit;
                [c(i,j),EE(i,j)]=fzero(f,c0,options); 
                % Solve bonds dec rule from budget constraint (and within
                % grid boundaries) 
                bp(i,j)=max((1/Q(i,j))*(yt(i,j)+b(i,j)-c(i,j)),bmin);
                bp(i,j)=min(bp(i,j),bmax); 
                % Here debt may not be consistent with consumption anymore
                % but it will be the optimal choice given the grid.       
            end
        end
    end
    price= (1-omega)/omega*c.^(1+ita);  % set PN to clear the N market
            
    %===============Consolidate new solutions==============================
    % To make sure c is feasible:
    c=b+yt-max(Q.*bp,-kappa*(price+yt));    % qb'>=-kappa*(yt+yn*pn)
    price= (1-omega)/omega*c.^(1+ita); % Price consistent with c
    bp=(1./Q).*(b+yt-c); % In principle this line should be redundant
    % End of Steps 3,4,5 of Cell 3 of the handout
    %========================================================================
 
    %%%Step 6 of Cell 3 of handout
    
    %=====================Updating rules for next iteration==============
    bp=uptd*bp+(1-uptd)*oldbp;
    c=uptd*c+(1-uptd)*oldc;
%    price=uptd*price+(1-uptd)*oldp;
    %========================================================================    
 
   %%%%%%%%%%%%%% Find new constrained values:%%%%%%%%%%%%%%%%%%%
    %%%%% This updates constrained dec rules for next iteration.
    bpmax=-kappa*(price.*yn +yt)./Q; %-(1./Q).*kappa.*(price+yt);
    bpmax(bpmax>bmax)=bmax;
    bpmax(bpmax<bmin)=bmin;
    cbind=b+yt-Q.*bpmax;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    iter=iter+1; % Update iteration counter
    
    %Calculate difference between new and old policies for convergence
    %check
    d2=max([max(max(abs(c-oldc))),max(max(abs(bp-oldbp)))]);
    
    % Print results once every (outfreq) iterations
    if mod(iter, outfreq) == 0;
        fprintf('%d          %1.7f \n',iter,d2);
    end
end
fprintf('%d          %1.7f \n',iter,d2);    % Print last iteration result
save MPPpoliciesDE.mat                      % Save DE results
toc
%% 4. Social Planner's Solution (SP) (Cell 4)

uptd=uptd/6;    % Slower updating speed of dec rule for planners problem 

%%%%%%%Initial conjectures for SP solution set at DE solutions or same as
%%%%%%%in DE case
bpSP=b; %bp;
cSP=b+yt-bpSP.*Q;  %c;
priceSP=(1-omega)/omega*cSP.^(1+ita); %price;

bpmaxSP=-kappa*(priceSP.*yn +yt)./Q;  %bpSP;
cbindSP=b+yt-bpmaxSP.*Q;  %c;
EESP=EE;
mupSP=ones(NB,Ntotal); % Preallocation for SP's marginal utility of tradables
emuSP=zeros(NB,Ntotal); % Preallocation for SP's exp. marginal utility
%%%%%%%

d2=100;      % Initialize difference between policies (just to enter loop)
iter=0;      % Initialize Iteration Counter
disp('SPP Iter     Norm');

while d2>tol && iter <iter_tol*2 %Exit if either convergence is attained.
    %or loop runs for more than number of times set in iter_tol parameter
    % times 2. Uses double because of slower updating
    
    % Set conjectures for current iteration
%    oldpSP=priceSP;
    oldcSP=cSP;
    oldbpSP=bpSP;
    
	% this is just (total.consumption)^(-ita)
    totalcSP=(omega*cSP.^(-ita)+(1-omega)*yn.^(-ita));     
	
    % Find marginal utility of tradables consumption
    mupSP= omega*totalcSP.^(sigma/ita).*(totalcSP.^(-1/ita-1)).*(cSP.^(-ita-1));  
	
    % Calculate externality term, Equation (8) in the handout:
    psi=(1-omega)/omega*(ita+1)*(cSP./yn).^ita*kappa;
    psibind=(1-omega)/omega*(ita+1)*(cbindSP./yn).^ita*kappa;
    % Calculate social marginal utility, equation (7) in the handout
    mupSP=EESP.*psi+mupSP;
    
    
    %%%% Find expected marginal utility tomorrow in today's grid, given
    %%%% current future debt policy bp(b,z). By interpolation
    parfor i=1:NB % % Can be parallelized using parfor instear of for
        for j=1:Ntotal
            emuSP(i,j)=beta*(1+R(i,j))*interp1(B,mupSP,bpSP(i,j),'linear','extrap')*Prob(j,:)';
			%EMU is expected marginal utility tomorrow as function of (b,z)
        end
    end
    
    parfor i=1:NB % % Can be parallelized using parfor instear of for
        for j=1:Ntotal
            
            %%%%%%% Step 2: Calculate Euler equation error at max debt.
            EESP(i,j)=((omega*cbindSP(i,j)^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cbindSP(i,j)^(-ita-1)-emuSP(i,j))./(1-psibind(i,j));
            cinitSP=cSP(i,j); %initial condition for solver
            if EESP(i,j)>tol_EEbind % Constraint binds: go to step 3
                
                %%%% Step 3
                bpSP(i,j)=bpmaxSP(i,j); % debt will be as big as possible
                cSP(i,j)=cbindSP(i,j);  % consumption is solved from budget constraint
               
            else % Debt restriction does not bind: go to step 4 in handout
                
                %%%%% Step 4 ,section 3 of handout:
                
                % Define function used to find consumption decision
                % that solves planner's Euler Equation 
                f = @(cc) (omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emuSP(i,j);
                % Solve consumption from Euler Equation
                c0=cinitSP;
                [cSP(i,j),EESP(i,j)]=fzero(f,c0,options); 
                % Find planner's bond decision and make sure is inside the grid
                bpSP(i,j)=max((1/Q(i,j))*(yt(i,j)+b(i,j)-cSP(i,j)),bmin);
                bpSP(i,j)=min(bpSP(i,j),bmax);
            end
        end
    end
    priceSP= (1-omega)/omega*cSP.^(1+ita);  % PN clears the N market
    %===============Check collateral constraint==============================
    % Again debt was forced into the grid but may not be consistent with
    % consumption.
    cSP=b+yt-max(Q.*bpSP,-kappa*(priceSP+yt));
    priceSP= (1-omega)/omega*cSP.^(1+ita);
    bpSP=(1./Q).*(b+yt-cSP);
    %========================================================================
    
    %=====================Updating rule.Must be slow, important!==============

%if d2<1e-2 % Smaller Updates as dec rules are closer to converging 
 %   uptd=0.001;
%else
 %   uptd=0.005;
%end  
   
    %=====================Updating rule==============
    bpSP=uptd*bpSP+(1-uptd)*oldbpSP;
    cSP=uptd*cSP+(1-uptd)*oldcSP;
%    priceSP=uptd*priceSP+(1-uptd)*oldpSP;
    %========================================================================
 
       %%%%%%%%% Find constrained values
     bpmaxSP=-(1./Q).*kappa.*(priceSP+yt);
     bpmaxSP(bpmaxSP>bmax)=bmax;
     bpmaxSP(bpmaxSP<bmin)=bmin;
     cbindSP=b+yt-Q.*bpmaxSP;

    %%%%%%%%%%%%%  
        
    % Determine distance between policies
    d2=max([max(max(abs(cSP-oldcSP))),max(max(abs(bpSP-oldbpSP)))]);
    
    iter=iter+1; %update iteration counter
    iter_sp = iter;
    d2_sp   = d2;
    if mod(iter, outfreq*5) == 0; %display every 5*outfreq results
        %fprintf('%d          %1.7f \n',iter,d2);
        fprintf('%d          %1.7f \n',iter,d2_sp);
        %fprintf('%d          %10.2e \n',iter_sp,d2_sp);
    end   
end
toc
%% 5. Welfare Calculation (Cell 5)

% Initiallize matrices for value functions of the Decentralize Equilibrium
% and Social Planner
V=zeros(NB,Ntotal);
VSP=zeros(NB,Ntotal);

% Flow utilities for each regime
U=(totalc.^(-1/ita)).^(1-sigma)./(1-sigma);
USP=(totalcSP.^(-1/ita)).^(1-sigma)./(1-sigma);

% Initialize expected utility at current flow utility
EV=U;
EVSP=USP;

d3=100;                         % Initial convergence criterion
iter=0;                         % Iteration counter
disp('Value Iter     Norm');    
while d3>tol && iter <iter_tol
    for i=1:NB
        for j=1:Ntotal
            % Calculate expected utility by interpolating current Value
            % function.
            EV(i,j)=beta*interp1(B,V,bp(i,j),'linear','extrap')*Prob(j,:)';
            EVSP(i,j)=beta*interp1(B,VSP,bpSP(i,j),'linear','extrap')*Prob(j,:)';  
        end
    end
    
    % New value function is flow utility plus NEW expected utility
    V_new=U+EV;
    VSP_new=USP+EVSP;
    
    % Check convergence for value functions
    d3=max([max(max(abs((V_new-V)./V))),max(max(abs((VSP_new-VSP)./VSP)))]);
    
    iter=iter+1;                    % Update counters
    V=V_new;                        % Uptade Value Functions
    VSP=VSP_new;
    if mod(iter, outfreq) == 0;
        fprintf('%d          %1.7f \n',iter,d3); % Display every (outfreq) iterations
    end
    
end
fprintf('%d          %1.7f \n',iter,d3); % Display last iteration

% Welfare gain of the SP over the DE:
Wf=(VSP./V).^(1/(1-sigma))-1;

%% 6. Optimal Tax (Cell 6)

totalcSP=(omega*cSP.^(-ita)+(1-omega)*yn.^(-ita)); 
mupSPtax= omega*totalcSP.^(sigma/ita).*(totalcSP.^(-1/ita-1)).*(cSP.^(-ita-1));  %mup(b,y)

% Find private marginal utility
%mupSPprivate=mupSP-EESP.*psi;
for i=1:NB
    for j=1:Ntotal
        % Here we calculate expected private marginal utility (note it is
        % already discounted by beta and R
       % emuSP(i,j)=beta*(1+R(i,j))*interp1(B,mupSPprivate,bpSP(i,j),'linear','extrap')*Prob(j,:)';
        emuSP(i,j)=beta*(1+R(i,j))*interp1(B,mupSPtax,bpSP(i,j),'linear','extrap')*Prob(j,:)';
    end
end
% Optimal tax: find the gross tax rate, equation (11) in the handout
Tau=(mupSP./emuSP-1);

Tau(EESP>0.001)=0;
Tau(Tau<0)=0;

%figure;
%plot(B,Tau); legend('1','2','3','4','5','6','7','8','9')
%xlabel('debt')

 %title('tax '); ylabel('%')
% Recall this is tax only for non-binding region for tax
% When unconstrained future debt increases with current debt
% Tax is set to 0 when constraint binds

AN=bpSP(2:end,:)-bpSP(1:end-1,:);   % change in future debt for adjacent current debt

for i=1:Ntotal
    % Find the first positive change, the first unconstrained debt choice
    IN(i)=find(AN(:,i)>0,1,'first'); 
end


for j=1:Ntotal
    % For all constrained, set tax to 1
    if IN(j)>1
        Tau(1:IN(j)-1,j)=0;
    end
end

% just in case, all taxes should be 0 or more.
Tau(Tau<0)=0;
save MPPpolicies.mat

%% 7. Figures 
% IR
pic=figure(4);
subplot(2,3,1)
plot(B(IN(1):end),Tau(IN(1):end,1),B(IN(1+NSS):end),Tau(IN(1+NSS):end,1+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Bad news,Bad Shocks')
xlim([bmin -0.6])
ylabel('Tax')

subplot(2,3,4)
plot(B(IN(3):end),Tau(IN(3):end,3),B(IN(3+NSS):end),Tau(IN(3+NSS):end,3+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Bad news, Good Shocks')
ylabel('Tax')
xlim([bmin -0.6])

subplot(2,3,2)
plot(B(IN(4):end),Tau(IN(4):end,4),B(IN(4+NSS):end),Tau(IN(4+NSS):end,4+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Avg news,Bad Shocks')
ylabel('Tax')
xlim([bmin -0.6])

subplot(2,3,5)
plot(B(IN(6):end),Tau(IN(6):end,6),B(IN(6+NSS):end),Tau(IN(6+NSS):end,6+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Avg news,Good Shocks')
ylabel('Tax')
xlim([bmin -0.6])

subplot(2,3,3)
plot(B(IN(7):end),Tau(IN(7):end,7),B(IN(7+NSS):end),Tau(IN(7+NSS):end,7+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Good news,Bad Shocks')
ylabel('Tax')
xlim([bmin -0.6])

subplot(2,3,6)
plot(B(IN(9):end),Tau(IN(9):end,9),B(IN(9+NSS):end),Tau(IN(9+NSS):end,9+NSS),':','linewidth',4)
legend('High R','Low R','location','northeast')
title('Good news,Good Shocks')
ylabel('Tax')
xlim([bmin -0.6])

% News type
pic=figure(5);
subplot(2,2,1)
plot(B(IN(1):end),Tau(IN(1):end,1),B(IN(4):end),Tau(IN(4):end,4),'--',B(IN(7):end),Tau(IN(7):end,7),':','linewidth',5)
title('Bad shock,High R')
xlabel('B')
ylabel('Tax')
xlim([bmin -0.6])
legend('Bad news','Average news','Good news','location','northeast')

subplot(2,2,2)
plot(B(IN(3):end),Tau(IN(3):end,3),B(IN(6):end),Tau(IN(6):end,6),'--',B(IN(9):end),Tau(IN(9):end,9),':','linewidth',5)
title('Good shock,High R')
xlabel('B')
ylabel('Tax')
xlim([bmin -0.6])
legend('Bad news','Average news','Good news','location','northeast')

subplot(2,2,3)
plot(B(IN(1+NSS):end),Tau(IN(1+NSS):end,1+NSS),B(IN(4+NSS):end),Tau(IN(4+NSS):end,4+NSS),'--',B(IN(7+NSS):end),Tau(IN(7+NSS):end,7+NSS),':','linewidth',5)
title('Bad shock,Low R')
xlabel('B')
ylabel('Tax')
xlim([bmin -0.6])
legend('Bad news','Average news','Good news','location','northeast')

subplot(2,2,4)
plot(B(IN(3+NSS):end),Tau(IN(3+NSS):end,3+NSS),B(IN(6+NSS):end),Tau(IN(6+NSS):end,6+NSS),'--',B(IN(9+NSS):end),Tau(IN(9+NSS):end,9+NSS),':','linewidth',5)
title('Good shock,Low R')
xlabel('B')
ylabel('Tax')
xlim([bmin -0.6])
legend('Bad news','Average news','Good news','location','northeast')

%% 8. Close parallel pool (comment this is in if using parallel toolbox)
if verLessThan('matlab','8.2')
	matlabpool close   % Old version. Use matlabpool
else
	delete(poolpbject) % Newer version, can use parpool
end
